unit DebugLogger;

interface

uses
  Mutex;

const
  LOG_NONE     = 0; // show even if level is zero
  LOG_MINOR    = 1; // show minor stuff
  LOG_FUNCS    = 2; // show function all function level
  LOG_MAJOR    = 3; // show major stuff
  LOG_PARANOID = 4; // show everything

type
  TDebugLogger = class
  private
    FFileName   : string;
    FOverwrite  : boolean;
    FMtx        : TMutex;
    FDebugLevel : integer;
    function GetThreadID : integer;
  public
    constructor Create(const FileName : string;
                       bOverwrite : boolean;
                       InitialDebugLevel : integer);
    destructor Destroy; override;
    procedure Log(LogLevel : integer; const Msg : string); overload;
    procedure Log(LogLevel : integer; const Msg: string; const Args: array of const); overload;
  published
    property DebugLevel : integer read FDebugLevel write FDebugLevel;
  end;

implementation

{$IFDEF LINUX}
uses
  SysUtils,Libc;
{$ENDIF}

{$IFDEF WIN32}
uses
  SysUtils,Windows;
{$ENDIF}

{ TDebugLogger }

constructor TDebugLogger.Create(const FileName: string;
  bOverwrite: boolean; InitialDebugLevel : integer);
begin
  inherited Create;
  FFileName   := FileName;
  FOverwrite  := bOverwrite;
  FDebugLevel := InitialDebugLevel;
  // create the mutex
  FMtx := TMutex.Create(maDefault);
end;

destructor TDebugLogger.Destroy;
begin
  // get rid of the mutex
  FMtx.Free;
  inherited;
end;

procedure TDebugLogger.Log(LogLevel : integer; const msg: string);
const
  DATEFORMAT = 'yyyy/mm/dd hh:nn:ss.zzz';

var
  f : TextFile;

begin
  // no use locking the mutex if we're not going to write this out..
  if LogLevel > FDebugLevel then
    Exit;

  // first lock the mutex
  FMtx.Lock;
  try
    AssignFile(F, FFileName);
    if (FileExists(FFileName)) and (not FOverwrite) then
      Append(F)
    else
      Rewrite(F);
    try
      writeln(F,FormatDateTime(DATEFORMAT,now),' ',GetThreadID,' ',msg);
    finally
      CloseFile(F);
    end;
  finally
    // unlock the mutex
    FMtx.UnLock;
  end;
end;

function TDebugLogger.GetThreadID: integer;
begin
{$IFDEF WIN32}
  Result := GetCurrentThreadID;
{$ENDIF}

{$IFDEF LINUX}
  Result := pthread_self;
{$ENDIF}
end;

procedure TDebugLogger.Log(LogLevel: integer; const Msg: string;
  const Args: array of const);
begin
  Log(LogLevel,Format(Msg,Args));
end;


end.
