unit Mutex;



interface



{$IFDEF LINUX}

uses

  SysUtils, IPCBase, Libc;

{$ENDIF}



{$IFDEF WIN32}

uses

  SysUtils, IPCBase, Windows;

{$ENDIF}



type

  {$IFDEF WIN32}

     TRTLCriticalSection = THandle;

  {$ENDIF}



  // the mutex attributes are only used on Linux.

  TMtxAttribute = ( maDefault,      // default mutex atributes

                    maFastNP,       // want a "fast" mutex (not portable)

                    maRecursiveNP); // want a Recursive mutex (not portable)



  TMutex = class(TIPCBase)

  private

    FMtx  : TRTLCriticalSection;

    FAttr : TMtxAttribute;

  public

    constructor Create(attr : TMtxAttribute);

    destructor  Destroy; override;

    procedure Lock;

    procedure UnLock;

    function TryLock : boolean;

  end;



{$IFDEF LINUX}

  TConditionVar = class(TIPCBase)

  private

    FCnd      : TCondVar;

    FMutex    : TMutex;

    bOwnMutex : boolean;

  public

    constructor Create(Mutex : TMutex);

    destructor  Destroy; override;

    procedure Wait;

    procedure Signal;     // signal only one waiting thread

    procedure Broadcast;  // signal all of the waiting threads

    function TimedWait(absTime : TTimeSpec) : boolean;

    property Mutex : TMutex read FMutex;

  end;

{$ENDIF}



implementation



{ TMutex }



{$IFDEF LINUX}

constructor TMutex.Create(attr : TMtxAttribute);

var

   mutexattr : TMutexAttribute;

   ret, attrvalue : integer;



begin

   inherited Create;

   FAttr := attr;



   // figure out what to do with the attributes..

   // currently only default attributes are used

   ret := pthread_mutexattr_init(mutexattr);

   if ret <> LIBC_SUCCESS then

      ErrorMessage('pthread_mutexattr_init');



   case attr of

      maFastNP      : attrvalue := PTHREAD_MUTEX_FAST_NP;

      maRecursiveNP : attrvalue := PTHREAD_MUTEX_RECURSIVE_NP;

   else

      attrvalue := PTHREAD_MUTEX_DEFAULT;

   end;



   ret := pthread_mutexattr_settype(mutexattr, attrvalue);

   if ret <> LIBC_SUCCESS then

      ErrorMessage('pthread_mutexattr_setttype');



   if pthread_mutex_init(FMtx,mutexattr) <> LIBC_SUCCESS then

      ErrorMessage('pthread_mutex_init');

{$ENDIF}



{$IFDEF WIN32}

constructor TMutex.Create(attr : TMtxAttribute);

begin

   inherited Create;

   FAttr := attr;

   // For Win32 mutexes, we will not worry about security, name, or initially

   // owning the mutex

   // Also attr is ignored

   FMtx := CreateMutex(nil,False,nil);

   if FMtx = 0 then

      ErrorMessage('CreateMutex');

{$ENDIF}

end;



{$IFDEF LINUX}

destructor TMutex.Destroy;

begin

   if pthread_mutex_destroy(FMtx) <> LIBC_SUCCESS then

      ErrorMessage('pthread_mutex_destroy');

   inherited;

end;

{$ENDIF}



{$IFDEF WIN32}

destructor TMutex.Destroy;

begin

   if not CloseHandle(FMtx) then

      ErrorMessage('CloseHandle');

   inherited;

end;

{$ENDIF}



{$IFDEF LINUX}

procedure TMutex.Lock;

var

   ret : integer;



begin

  ret := pthread_mutex_lock(FMtx);

  if (ret <> LIBC_SUCCESS) then

     ErrorMessage('pthread_mutex_lock');

end;

{$ENDIF}



{$IFDEF WIN32}

procedure TMutex.Lock;

var

   ret : DWORD;



begin

  ret := WaitForSingleObject(FMtx,INFINITE);

  if (ret <> WAIT_OBJECT_0) then

     ErrorMessage('WaitForSingleObject');

end;

{$ENDIF}



{$IFDEF LINUX}

function TMutex.TryLock: boolean;

var

  ret : integer;

begin

  ret := pthread_mutex_trylock(FMtx);

  Result := (ret = LIBC_SUCCESS);

end;

{$ENDIF}



{$IFDEF WIN32}

function TMutex.TryLock: boolean;

var

  ret : DWORD;

begin

  ret := WaitForSingleObject(Fmtx,0);

  Result := (ret = WAIT_OBJECT_0);

end;

{$ENDIF}



{$IFDEF LINUX}

procedure TMutex.UnLock;

var

   ret : integer;

begin

  ret := pthread_mutex_unlock(FMtx);

  if (ret <> LIBC_SUCCESS) then

     ErrorMessage('pthread_mutex_unlock');

end;

{$ENDIF}



{$IFDEF WIN32}

procedure TMutex.UnLock;

begin

  if not ReleaseMutex(FMtx) then

     ErrorMessage('ReleaseMutex');

end;

{$ENDIF}



{ TConditionVar }



{$IFDEF LINUX}

procedure TConditionVar.Broadcast;

var

  ret : integer;



begin

  ret := pthread_cond_broadcast(FCnd);

  if ret <> LIBC_SUCCESS then

     ErrorMessage('pthread_cond_broadcast');

end;



constructor TConditionVar.Create(Mutex : TMutex);

var

   ret : integer;



begin

   inherited Create;

   // create a mutex to be associated with this condition var

   if not Assigned(Mutex) then

   begin

     bOwnMutex := true;

     FMutex := TMutex.Create(maDefault);

   end

   else

   begin

     bOwnMutex := false;

     FMutex := Mutex;

   end;



   // initialize the condition variable

   ret := pthread_cond_init(FCnd,nil);

   if ret <> LIBC_SUCCESS then

      ErrorMessage('pthread_cond_init');

end;



destructor TConditionVar.Destroy;

var

  ret : integer;



begin

  // first thing, free up the mutex associated with this cond var.

  // but only if we created it

  if bOwnMutex then

    FMutex.Free;

  // free the condition variable

  ret := pthread_cond_destroy(FCnd);

  if ret <> LIBC_SUCCESS then

     ErrorMessage('pthread_cond_init');

  inherited;

end;



procedure TConditionVar.Signal;

var

  ret : integer;



begin

  ret := pthread_cond_signal(FCnd);

  if ret <> LIBC_SUCCESS then

     ErrorMessage('pthread_cond_signal');

end;



function TConditionVar.TimedWait(absTime: TTimeSpec) : boolean;

var

  ret : integer;



begin

  ret := pthread_cond_timedwait(FCnd,FMutex.FMtx,absTime);

  Result := (ret = LIBC_SUCCESS);

end;



procedure TConditionVar.Wait;

var

  ret : integer;



begin

  ret := pthread_cond_wait(FCnd,FMutex.FMtx);

  if ret <> LIBC_SUCCESS then

     ErrorMessage('pthread_cond_wait');

end;



{$ENDIF}



end.

